@@ -6,7 +6,7 @@ from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Tuple
-from typing import Union
+from requests.exceptions import ConnectionError
 
 from .connection import CloudflareScraper
 from .connection import Response
@@ -15,12 +15,21 @@ from .connection import get_binary_raw
 from .connection import get_robots
 from .connection import join_url
 from .connection import make_session
+from .journal import Journal
 from .parse import BeautifulSoup
 from .parse import parse_page
 from .sub import Sub
 from .sub import SubPartial
 
 
+class DisallowedPath(Exception):
+    pass
+
+
+class CrawlDelayError(Exception):
+    pass
+
+
 class FAAPI:
     def __init__(self, cookies: List[Dict[str, Any]] = None):
         self.cookies: List[Dict[str, Any]] = [] if cookies is None else cookies
@@ -37,14 +46,21 @@ class FAAPI:
     def handle_delay(self):
         if (delay_diff := perf_counter() - self.last_get) < self.crawl_delay:
             if self.raise_for_delay:
-                raise Exception(f"Crawl-delay not respected {delay_diff}<{self.crawl_delay}")
+                raise CrawlDelayError(f"Crawl-delay not respected {delay_diff}<{self.crawl_delay}")
             sleep(self.crawl_delay - delay_diff)
         self.last_get = perf_counter()
 
     def check_path(self, path: str):
         for pattern in self.robots.get("disallow", []):
             if re_search(pattern.replace("*", ".*"), "/" + path.lstrip("/")):
-                raise Exception(f"Path {path} is not allowed by robots.txt {pattern}")
+                raise DisallowedPath(f"Path {path} is not allowed by robots.txt {pattern}")
+
+    @property
+    def connection_status(self) -> bool:
+        try:
+            return self.get("/").ok
+        except ConnectionError:
+            return False
 
     def get(self, path: str, **params) -> Response:
         self.check_path(path)
@@ -58,7 +74,7 @@ class FAAPI:
     def get_sub(self, sub_id: int, get_file: bool = False) -> Tuple[Sub, Optional[bytes]]:
         assert isinstance(sub_id, int) and sub_id > 0
 
-        sub_page = self.get_parse(f"/view/{sub_id}")
+        sub_page = self.get_parse(join_url("view", sub_id))
         sub = Sub(sub_page)
         sub_file = self.get_sub_file(sub) if get_file else None
 
@@ -68,20 +84,27 @@ class FAAPI:
         self.handle_delay()
         return get_binary_raw(self.session, sub.file_url)
 
+    def get_journal(self, journal_id: int) -> Journal:
+        assert isinstance(journal_id, int) and journal_id > 0
+
+        journal_page = self.get_parse(join_url("journal", journal_id))
+        return Journal(journal_page=journal_page)
+
     def userpage(self, user: str) -> Tuple[str, str, Optional[BeautifulSoup]]:
         assert isinstance(user, str)
 
-        page: Optional[BeautifulSoup] = self.get_parse(join_url("user", user))
-        if page is None:
+        page_parsed: Optional[BeautifulSoup] = self.get_parse(join_url("user", user))
+
+        if page_parsed is None:
             return "", "", None
 
-        username_div = page.find("div", class_="username")
+        username_div = page_parsed.find("div", class_="username")
 
         username = username_div.find("span").text.strip()
         status = username[0]
         username = username[1:]
 
-        description = page.find("div", class_="userpage-profile").text.strip()
+        description = page_parsed.find("div", class_="userpage-profile").text.strip()
 
         return username, status, parse_page(description)
 
@@ -89,7 +112,7 @@ class FAAPI:
         assert isinstance(user, str)
         assert isinstance(page, int) and page >= 1
 
-        page_parsed = self.get_parse(join_url("gallery", user, str(page)))
+        page_parsed = self.get_parse(join_url("gallery", user, page))
 
         if page_parsed is None or page_parsed.title.text.lower().startswith("account disabled"):
             return [], 0
@@ -102,7 +125,7 @@ class FAAPI:
         assert isinstance(user, str)
         assert isinstance(page, int) and page >= 1
 
-        page_parsed = self.get_parse(join_url("scraps", user, str(page)))
+        page_parsed = self.get_parse(join_url("scraps", user, page))
 
         if page_parsed is None or page_parsed.title.text.lower().startswith("account disabled"):
             return [], 0
@@ -127,16 +150,31 @@ class FAAPI:
 
         return list(map(SubPartial, subs)), page_next
 
+    def journals(self, user: str, page: int = 1) -> Tuple[List[Journal], int]:
+        assert isinstance(user, str)
+        assert isinstance(page, int)
+
+        page_parsed = self.get_parse(join_url("journals", user, page))
+        username = page_parsed.find("div", class_="username").find("span").text.strip()[1:]
+
+        journals_tags = page_parsed.findAll("section")
+        journals = list(map(Journal, journals_tags))
+        for j in journals:
+            j.author = username
+
+        return journals, (page + 1) if journals else 0
+
     def search(self, q: str, page: int = 1, **params) -> Tuple[List[SubPartial], int, int, int, int]:
         assert isinstance(q, str)
         assert isinstance(page, int)
 
         page_parsed = self.get_parse("search", q=q, page=page, **params)
 
-        subs = page_parsed.findAll("figure")
         if page_parsed is None:
             return [], 0, 0, 0, 0
 
+        subs = page_parsed.findAll("figure")
+
         query_stats = page_parsed.find("div", id="query-stats")
         for div in query_stats("div"):
             div.decompose()
@@ -146,10 +184,10 @@ class FAAPI:
 
         return list(map(SubPartial, subs)), page_next, a, b, tot
 
-    def user_exists(self, user: str):
+    def user_exists(self, user: str) -> bool:
         assert isinstance(user, str)
 
-        res = self.get(join_url("view", user))
+        res = self.get(join_url("user", user))
 
         if not res.ok:
             return False
@@ -162,10 +200,24 @@ class FAAPI:
         else:
             return True
 
-    def sub_exists(self, sub_id: Union[int, str]):
-        assert isinstance(sub_id, int) or (isinstance(sub_id, str) and sub_id.isdigit())
+    def sub_exists(self, sub_id: int) -> bool:
+        assert isinstance(sub_id, int) and sub_id > 0
 
-        res = self.get(join_url("view", str(sub_id)))
+        res = self.get(join_url("view", sub_id))
+
+        if not res.ok:
+            return False
+        elif not (title := parse_page(res.text).title.text):
+            return False
+        elif title.text.lower() == "system error":
+            return False
+        else:
+            return True
+
+    def journal_exists(self, journal_id: int) -> bool:
+        assert isinstance(journal_id, int) and journal_id > 0
+
+        res = self.get(join_url("journal", journal_id))
 
         if not res.ok:
             return False
